@@ -76,6 +76,7 @@ typedef struct move
 static bool  g_useColor = true;
 static bool  g_playing = false;
 static bool  g_useHighlights = false;
+static bool  g_output_stream = false;		/* is the -o flag set */
 static char* g_provided_stream = NULL;		/* in case of -s flag the user provides the sudoku stream */
 static bool  g_resume_game = false;         /* in case of -r flag and saved game state */
 static int   g_resume_level;                /* store difficulty of the resume game */
@@ -109,6 +110,7 @@ static void print_usage(void)
 	printf(_("-h help:\t\tPrint this help\n"));
 	printf(_("-v version:\t\tPrint version\n"));
 	printf(_("-c nocolor:\t\tDo not use colors\n"));
+	printf(_("-o output:\t\tOutput stream (inverse of -s)\n"));
 	printf(_("-d difficulty:\t\tChoose between: easy, normal, hard\n"));
 	printf(_("-s stream:\t\tUser provided sudoku stream\n"));
 	printf(_("-r resume:\t\tResume the last saved game\n"));
@@ -273,10 +275,18 @@ bool save_stream(char user_board[], char plain_board[], int n)
 	return true;
 }
 
+void generate_stream_output(int difficulty) {
+	char* stream = generate_puzzle(difficulty);
+
+	printf("%s\n", stream);
+
+	free(stream);
+}
+
 static void parse_arguments(int argc, char *argv[])
 {
 	int opt;
-	while ((opt = getopt(argc, argv, "hvcrs:d:p:i:n:")) != -1)
+	while ((opt = getopt(argc, argv, "hvcors:d:p:i:n:")) != -1)
 	{
 		switch (opt)
 		{
@@ -289,6 +299,9 @@ static void parse_arguments(int argc, char *argv[])
 			case 'c':
 				g_useColor = false;
 				break;
+			case 'o':
+				g_output_stream = true;
+				break;
 			case 's':
 				if (!is_valid_stream(optarg))
 					exit(EXIT_FAILURE);
@@ -643,6 +656,12 @@ int main(int argc, char *argv[])
 	parse_arguments(argc, argv);
 	srand(time(NULL));
 
+	if (g_output_stream)
+	{
+		generate_stream_output(g_level);
+		return EXIT_SUCCESS;
+	}
+
 	if (g_outputFilename)
 	{
 #ifdef ENABLE_CAIRO
@@ -696,51 +715,64 @@ int main(int argc, char *argv[])
 		{
 			case 'h':
 			case KEY_LEFT:
-				if(x>5)
+				if (x-GRID_NUMBER_START_X <= 0) {
+					x = GRID_NUMBER_START_X + GRID_LINE_DELTA*8;
+				}
+				else if(x>5)
 				{
 					x -= GRID_LINE_DELTA;
-					if(g_playing)
-					{
-						// if we have highlighting enabled, we need to redraw the whole grid
-						// so we can have the new colors in the matching colors.
-						// this should only be done when we are playing, because plain_board
-						// is actually the one being solved and thus displayed.
-						// this is true for all movement keys.
-						fill_grid(user_board, plain_board, x, y);
-					}
+
+				}
+				if(g_playing)
+				{
+					// if we have highlighting enabled, we need to redraw the whole grid
+					// so we can have the new colors in the matching colors.
+					// this should only be done when we are playing, because plain_board
+					// is actually the one being solved and thus displayed.
+					// this is true for all movement keys.
+					fill_grid(user_board, plain_board, x, y);
 				}
 				break;
 			case 'l':
 			case KEY_RIGHT:
-				if(x<34)
+				if ((x-GRID_NUMBER_START_X) / GRID_LINE_DELTA >= 8) {
+					x = GRID_NUMBER_START_X;
+				}
+				else if(x<34)
 				{
 					x += GRID_LINE_DELTA;
-					if(g_playing)
-					{
-						fill_grid(user_board, plain_board, x, y);
-					}
+				}
+				if(g_playing)
+				{
+					fill_grid(user_board, plain_board, x, y);
 				}
 				break;
 			case 'k':
 			case KEY_UP:
-				if(y>2)
+				if (y-GRID_NUMBER_START_Y <= 0) {
+					y = GRID_NUMBER_START_Y + GRID_COL_DELTA*8;
+				}
+				else if(y>2)
 				{
 					y -= GRID_COL_DELTA;
-					if(g_playing)
-					{
-						fill_grid(user_board, plain_board, x, y);
-					}
+				}
+				if(g_playing)
+				{
+					fill_grid(user_board, plain_board, x, y);
 				}
 				break;
 			case 'j':
 			case KEY_DOWN:
-				if(y<17)
+				if ((y-GRID_NUMBER_START_Y) / GRID_COL_DELTA >= 8) {
+					y = GRID_NUMBER_START_Y;
+				}
+				else if(y<17)
 				{
 					y += GRID_COL_DELTA;
-					if(g_playing)
-					{
-						fill_grid(user_board, plain_board, x, y);
-					}
+				}
+				if(g_playing)
+				{
+					fill_grid(user_board, plain_board, x, y);
 				}
 				break;
 			case 'Q':
